Package: ncGTW
Type: Package
Title: Alignment of LC-MS Profiles by Neighbor-wise Compound-specific
        Graphical Time Warping with Misalignment Detection
Version: 1.27.0
Author: Chiung-Ting Wu <ctwu@vt.edu>
Maintainer: Chiung-Ting Wu <ctwu@vt.edu>
biocViews: Software, MassSpectrometry, Metabolomics, Alignment
Description: The purpose of ncGTW is to help XCMS for LC-MS data alignment. 
    Currently, ncGTW can detect the misaligned feature groups by XCMS, and the
    user can choose to realign these feature groups by ncGTW or not.
License: GPL-2
Encoding: UTF-8
LazyData: true
RoxygenNote: 6.1.1
LinkingTo: Rcpp
Suggests: BiocStyle, knitr, testthat, rmarkdown
VignetteBuilder: knitr
Depends: methods, BiocParallel, xcms
Imports: Rcpp, grDevices, graphics, stats
BugReports: https://github.com/ChiungTingWu/ncGTW/issues
git_url: https://git.bioconductor.org/packages/ncGTW
git_branch: devel
git_last_commit: 87c9be4
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-07-14
NeedsCompilation: yes
Packaged: 2026-07-14 23:15:40 UTC; biocbuild
