# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

graphCut <- function(termW, edges) {
    .Call('_ncGTW_graphCut', PACKAGE = 'ncGTW', termW, edges)
}

