setGeneric("groupInfo", function(object, ...) standardGeneric("groupInfo"))
setGeneric("profiles", function(object, ...) standardGeneric("profiles"))
setGeneric("rtRaw", function(object, ...) standardGeneric("rtRaw"))

setGeneric("alignData", function(object, ...) standardGeneric("alignData"))
setGeneric("scanRange", function(object, ...) standardGeneric("scanRange"))
setGeneric("ncGTWpath", function(object, ...) standardGeneric("ncGTWpath"))
setGeneric("downSample", function(object, ...) standardGeneric("downSample"))

setGeneric("rtncGTW", function(object, ...) standardGeneric("rtncGTW"))
setGeneric("ncGTWpeaks", function(object, ...) standardGeneric("ncGTWpeaks"))
